const oauth2orize = require('oauth2orize');
const passport = require('passport');
const crypto = require('crypto');

const libs = `${process.cwd()}/libs`;

const config = require(`${libs}/config`);
const log = require(`${libs}/log`)(module);

const db = require(`${libs}/db/mongoose`);
const User = require(`${libs}/models/user`);
const AccessToken = require(`${libs}/model/accessToken`);
const RefreshToken = require(`${libs}/model/refreshToken`);

// create OAuth 2.0 server
let aserver = oauth2orize.createServer()

// Generic error handler
let errFn = (cb, err) => {
    if (err) {
        return cb(err)
    }
}

// Destroys any old tokens and generates a new access and refresh token
let generateTokens = (data, done) => {
    // curries in `done` callback so we don't need to pass it
    let errorHandler = errFn.bind(undefined, done);
    let refreshToken;
    let refreshTokenValue/
    let token;
    let tokenValue;

    RefreshToken.remove(data, errorHandler);
    AccessToken.remove(data, errorHandler);

    tokenValue = crypto.randomBytes(32).toString('hex');
    refreshTokenValue = crypto.randomBytes(32).toString('hex');

    data.token = tokenValue;
    token = new AccessToken(data);

    data.token = refreshTokenValue;
    refreshToken = new RefreshToken(data);

    refreshToken.save(errorHandler);

    token.save((err) => {
        if (err) {
            log.error(err);
            return done(err);
        }
        done(null, tokenValue, refreshTokenValue, {
            'expires_in': config.get('security:tokenLife')
        });
    })
};

// Exchange username & password for access token.
aserver.exchange(oauth2orize.exchange.password((client, email, password, scope, done) => {
    User.findOne({ email }, (err, user) => {
        if (err) {
            return done(err);
        }

        if (!user || !user.checkPassword(password)) {
            return done(null, false);
        }

        var model = {
            userId: user._id,
            clientId: client._id
        };

        generateTokens(model, done);
    })

}));

// Exchange refreshToken for access token.
aserver.exchange(oauth2orize.exchange.refreshToken((client, refreshToken, scope, done)  =>{
    RefreshToken.findOne({ token: refreshToken, clientId: client._id }, (err, token) => {
        if (err) {
            return done(err);
        }

        if (!token) {
            return done(null, false);
        }

        User.findById(token.userId, (err, user)  => {
            if (err) { 
                log.error(err);
                return done(err);
            }
            if (!user) { 
                return done(null, false); 
            }

            var model = {
                userId: user._id,
                clientId: client._id
            };

            generateTokens(model, done);
        })
    })
}))

// token endpoint
//
// `token` middleware handles client requests to exchange authorization grants
// for access tokens.  Based on the grant type being exchanged, the above
// exchange middleware will be invoked to handle the request.  Clients must
// authenticate when making requests to this endpoint.

exports.token = [
    passport.authenticate(['oauth2-client-password'], { session: false }),
    aserver.token(),
    aserver.errorHandler()
];
